/***********************************************************
 * laydir.h -- header file for dirlay.c		  	   *
 * Jun Yang, November 17, 1994				   *
 ***********************************************************/

/*---------------- Types and Classes ---------------*/
class _DI_Node ;

struct _DI_nbnode {
    _DI_Node*		id ;
    FLAG		fReversed ;
    BasicEdge		*edge_ptr ;
} ;

typedef DoubleList<_DI_nbnode*>	_DI_NBLIST ;
typedef BasicNode* _DI_NODEID ;

class _DI_Node {
    _DI_NODEID		id ;
    _DI_NBLIST*		nblist ;
    _DI_NBLIST*		nblist_in ;
    int			lambda ;
    int			layer ;
    bool		dummy ;
    FLAG		visited ;
    int			x ;
    int			y ;
    int			width ;
    int			height ;

    public:

	// some public variables
	int		Indegree ;
	int		Outdegree ;
	int		InPriority ;
	int		OutPriority ;
	int		top_label ;
	int		order ;
	double		avg_x ;

        _DI_Node(_DI_NODEID) ;                
        ~_DI_Node(void) ;

        _DI_NODEID      getID(void) ;
        _DI_NBLIST*     getNBList(void) ;
        _DI_NBLIST*     getNBList_in(void) ;
        int	        getLambda(void) ;
	bool		setLambda(int) ;
	int		getLayer(void) ;
	bool		setLayer(int) ;
	bool		isDummy(void) ;
	void		setDummy(bool) ;
	bool		isVisited(void) ;
	void		markVisited(bool) ;
	int		getX(void) ;
	void		setX(int) ;
	int		getY(void) ;
	void		setY(int) ;
        int             getWidth(void) ;
        void            setWidth(int) ;
        int	        getHeight(void) ;
        void            setHeight(int) ;

	// some operations
	_DI_nbnode*	searchNB_in(_DI_Node*) ;
} ;
    
typedef DoubleList<_DI_Node*> _DI_NODELIST ;

class _DI_Graph {
	_DI_NODELIST*	node_list ;

    public:
        _DI_Graph(void) ;
        ~_DI_Graph(void) ;

	// access methods
	_DI_NODELIST* getNodeList(void) ;

        // operations on node_list
        _DI_Node* searchNode(_DI_NODEID) ;
        bool insertNode(_DI_NODEID) ;
        bool deleteNode(_DI_NODEID) ;
        bool insertEdge(_DI_NODEID, _DI_NODEID) ;
        bool deleteEdge(_DI_NODEID, _DI_NODEID) ;
        void insertNodeEdge(_DI_NODEID, _DI_NODEID) ;

        // operations on neighbour node list
        _DI_nbnode* searchNBNode(_DI_Node*, _DI_Node*) ;
        bool     insertNBNode(_DI_Node*, _DI_nbnode*) ;
        bool     removeNBNode(_DI_Node*, _DI_nbnode*) ;

	// miscellaneous
        void	 markVisitedFlags(bool) ;
} ;

/*--------------- Function Prototypes ---------------*/

bool DI_layout(BasicGraph*) ;
void consAdjLists(BasicGraph*, _DI_Graph*) ;
void Simple_FAS(_DI_Graph*) ;
void simple_dfs(_DI_Node*) ;
void double_dfs(_DI_Node*) ;
void calIndegree(_DI_Graph*) ;
void calOutdegree(_DI_Graph*) ;
void calInPriority(_DI_Graph*) ;
void calOutPriority(_DI_Graph*) ;
void sortLayerUpPriority(void) ;
void sortLayerDownPriority(void) ;
void LongestPathLayer(_DI_Graph*) ;
void toplogicalSort(_DI_Graph*) ;
void addDummyNodes(_DI_Graph*) ;
void compMinPageWidth(_DI_Graph*) ;
void barycenterSort(_DI_Graph*) ;
void assignDiPos(_DI_Graph*) ;
void edgepos_dfs(_DI_Node*) ;
void restoreEdgeDir(_DI_Graph*) ;
bool _DIGRAPH_edge_ppath(BasicGraph*) ;
void init_order(_DI_Graph*) ;
void init_pos(void) ;
void set_one_layer_pos(int layer) ;
void transpose(_DI_Graph*) ;
int crossing_down(_DI_Node*, _DI_Node*) ;
void nodeordering(_DI_Graph*) ;
bool assignLabelRPos(BasicGraph*, _DI_Graph*) ;
bool assignLabelPPos(BasicGraph*) ;
bool _DI_calboxsize(BasicGraph*, _DI_Graph*) ;

/***********************************************************
 * End of laydir.h    					   *
 ***********************************************************/
