#include <string.h>
#include "stack.h"

template<class T>
Stack<T>::Stack()
{
    height = 0;
    allocateError = false;
    top = NULL;
}

template<class T>
Stack<T>& Stack<T>::copy(Stack<T>& g)
{
    StackNode *p, *q, *r;

    // clear the stack if not already empty
    if (top)
        clear();
    // copy the stack heights
    height = g.height;
    allocateError = false;
    // assign NULL to top of the target stack
    top = NULL;
    // exit if the source stack is empty
    if (!g.top)
        return *this;
    // allocate a new stack element
    top = new StackNode;
    // exit if error in allocation
    if (!top) {
        allocateError = true;
        return *this;
    }
    // initialize pointers for deep copy
    top->nextPtr = NULL;
    top->nodeData = g.top->nodeData;
    q = g.top->nextPtr;
    p = top;
    // loop to duplicate the source stack elements
    while (q) {
        // create new stack element
        r = new StackNode;
        if (!r) {
            allocateError = true;
            return *this;
        }
        r->nodeData = q->nodeData; // copy data
        r->nextPtr = NULL;
        p->nextPtr = r;
        p = p->nextPtr;
        q = q->nextPtr;
    }
    return *this;
}


template<class T>
void Stack<T>::clear()
{
    T x;

    while (pop(x))
	/* do nothing */;
}

template<class T>
void Stack<T>::push(T x)
{
    StackNode *p;

    allocateError = false;
    if (top) {
        p = new StackNode; // allocate new stack element
        if (!p) {
	    allocateError = true;
	    return;
	}
        p->nodeData = x;
	p->nextPtr = top;
	top = p;
    }
    else {
        top = new StackNode;
	if (top == NULL) {
	    allocateError = true;
	    return;
	}
	top->nodeData = x;
	top->nextPtr = NULL;
    }
    height++;
}

template<class T>
bool Stack<T>::pop(T& x)
{
    StackNode *p;

    if (height) {
        x = top->nodeData;
	p = top;
	top = top->nextPtr;
	delete p; // deallocate stack node
	height--;
	return true;
    }
    else
	return false;
}


