/*********************************************************************
 * swan.h ---- the general header file for Swan.	                 *
 * Jun Yang, Nov.17, 1994				                             *		
 *********************************************************************
 *  NOTE:  1) 1/25/97 - changed the var name LCID to LC_ID to avoid  *
 *                      complications with win95's LCID call.  jrf   *
 *         2) 1/26/97 - changed any references to BOOLEAN to BOOL to *
 *                      avoid conflicts with predefined BOOLEAN under*
 *                      Visual C++.  jrf                             *
 *********************************************************************/
 
/*--------------- Header Files ---------------*/
#ifndef _SWAN_H
#define _SWAN_H


#include "comndata.h"
#include "list.h"
#include "list.cpp"
#include "stack.h"
#include "stack.cpp"
#include "swerror.h"

//#ifdef SAILPP			removed because all modules are done in C++
#include "sail.h"
//#else  
//extern "C" {
//#include "sail.h"
//}
//#endif

#include <fstream.h>
//#include <libc.h>

#include "class.h"

/*--------------- Constants ---------------*/
 
// node shapes
#define RECTNODE	1
#define CIRCLENODE	2

// constant numbers
#define MAXGRAPH	128	// maximum graphs allowed in Swan
#define MAXNODE		4096	// maximum nodes allowed in Swan
#define NUMVERTEX	128

// screen coordinates
#define SCREENLEFTBORDER	0
#define SCREENRIGHTBORDER	32000
#define SCREENTOPBORDER		0
#define SCREENBOTTOMBORDER	24000
#define MAX_XCOORD		32767
#define MAX_YCOORD		32767
#define SCREENXOFFSET		200
#define SCREENYOFFSET		160

#define MINDISTX		10
#define MINDISTY		8
#define MINEDGELEN		10
#define NODEOFFSETX 		15
#define NODEOFFSETY 		10
#define XOFFSETTEXT 		3
#define YOFFSETTEXT 		12
#define GRAPHSTARTX 		100
#define GRAPHSTARTY 		100
#define ZOOMSTEP        	0.4
#define ZOOMUPPERBOUND  	4.0
#define ZOOMLOWERBOUND  	0.2
#define ZOOMOUTSTR		0.61

#define RECTNODEWIDTH   	27 
#define RECTNODEHEIGHT  	14
#define LONGRECTNODEWIDTH 	48
#define CIRCLENODERADIUS 	8

#define MINLCMARGIN		10

// fonts information
#define CURFONTWIDTH		6
#define CURFONTHEIGHT   	9	
#define TEXTHORIPADD		6
#define TEXTVERTPADD		3
#define TEXTLENLIMIT		3

#define MAXBUSLINE	10
#define BUSDIST		2

// miscellaneous
#define Y_SEPARATION    20

// edge orientation
#define HORIZONTAL	1
#define VERTICAL	2
#define DIAGONAL	3
#define VICEDIAGONAL	4

/*--------------- Types and Classes ---------------*/

typedef DL_Point 		CURVE ;
typedef DL_nbnode 		NODENBLIST ;
typedef DL_BasicNode		NODELIST ;
typedef DL_BasicEdge		EDGELIST ;
typedef DL_BasicLayoutComp 	LCLIST ;
typedef DL_BasicLayoutComp 	LCNBLIST ;
typedef DL_BasicGraph		GRAPHLIST ;

typedef int	ARROWTYPE ;
typedef int	LINETYPE ;
typedef int	COLORTYPE ;
typedef int	PICKTYPE ;
typedef int	COUNTER ;

struct SWPOINT {
	int	x ;	// x-coordinate
	int 	y ;	// y-coordinate

	bool operator==(SWPOINT p1) {
	    return (p1.x == x && p1.y == y) ? TRUE : FALSE ; }
	bool operator!=(SWPOINT p1) {
	    return (p1.x != x || p1.y != y) ? TRUE : FALSE ; }
} ;

typedef SWPOINT LOCATION ;
// typedef DoubleList<SWPOINT> CURVE ;

typedef struct {
	int	w ;	// width
	int	h ;	// height
} BOXSIZE ;

typedef struct {
	NODETYPE	node_type ; // default node type in this graph
	FLAG		directed ; // TRUE--directed, FALSE--undirected
	int		distX ; // distance between two neighbours in X
	int		distY ; // distance between two neighbours in Y
	FLAG		fShowLabel ;
} GRAPHINFO ;

typedef struct {
	int		shape ;
	BOXSIZE		size ;
	int		offsetX ; // X coord of node label 
	int		offsetY ; // Y coord of node label
	COLORTYPE 	color ;
/*	bool 	filled ; // filled or framed */
	int 	filled ; // filled or framed
	int		lineTH ; // thin or thick line
} NODEINFO ;

typedef struct {
	int		length ;
	LINETYPE	line_type ;
	int		lineTH ;
	ARROWTYPE	arrow_type ;
	COLORTYPE	color ;
	LOCATION	label_ppos ;
	LOCATION	label_wpos ;
	FLAG		label_orient ;
} EDGEINFO ;

typedef struct {
	int	left_margin ;
	int	right_margin ;
	int	top_margin ;
	int	bottom_margin ;
	int	distX ;
	int	distY ;
} LCINFO ;
	
typedef BasicLayoutComp* LAYOUTCOMPID ;
typedef int EDGEID ;

struct _nb_node {
	BasicNode	*id ;
	BasicEdge	*edge_ptr ;
	FLAG		fChild ;
} ;

//typedef PDoubleList<struct _nb_node*> NODENBLIST ;
typedef BasicNode* INNODEID ;

class BasicNode {
        NODEID  	id ;	// external id, used by the annotated
				// program to access this node
        LAYOUTCOMPID	lc_id ; // the layout component contains this
				// node
	LOCATION 	rpos ;	// relative position of this node
				// in its layout component
        LOCATION    	ppos ;  // physical position of the node 
				// in display plane
	LOCATION	wpos ;  // the position in Swan display window
	BOXSIZE		wsize ; // the box size of the node in window

        NODETYPE       	type ;   // type of the node
        LABEL          	label ;  // text in the node
        FLAG           	active ; // display status
	FLAG		traversed ; // traversal flag
        NODEINFO       	*info ;  // display attributes

	NODENBLIST	*nblist ; // neighbour list

	bool	       	rbus[MAXBUSLINE] ;
	bool	    	bbus[MAXBUSLINE] ;

    friend class BasicEdge ;
    friend class BasicGraph ;
    friend class BasicLayoutComp ;

    public:
	int		g_index ;

	// constructors and destructors
        BasicNode(NODETYPE, NODEID, LABEL) ;
	BasicNode(void) ;
        ~BasicNode(void) ;

	// access member functions
	NODEID		getID(void) ;
	NODENBLIST*	getNBList(void) ;
	bool		setLayoutComp(LAYOUTCOMPID) ;
	LAYOUTCOMPID	getLayoutComp(void) ;
        bool         setAttr(NODEINFO*) ;
	NODEINFO*	getAttr(void) ;
        bool         setRPos(LOCATION) ;
        LOCATION        getRPos(void) ;
        bool         setPPos(LOCATION) ;
        LOCATION        getPPos(void) ;
        bool         setColor(COLORTYPE) ;
        COLORTYPE       getColor(void) ;
        bool         setBoxSize(BOXSIZE) ;
        BOXSIZE         getBoxSize(void) ;
	bool		setFilled(FLAG) ;
	FLAG 		getFilled(void) ;
	bool		setType(NODETYPE) ;
	NODETYPE	getType(void) ;
	bool		setWidth(int) ;
	int		getWidth(void) ;
	bool		setHeight(int) ;
	int		getHeight(void) ;
	bool		setLineTH(int) ;
	int		getLineTH(void) ;
	bool		setLabel(LABEL) ;
	LABEL		getLabel(void) ;	
	int		getRBus(void) ;
	int		getBBus(void) ;
	void		setTravFlag(void) ;
	void		clearTravFlag(void) ;

	// node states
	FLAG   		isDisplayed(void) ;
	void		enActive(void) ;
	void		disActive(void) ;
	bool		isTraversed(void) ;

	// manipulation of nodes and edges
	BasicEdge*	getEdge(BasicNode*) ;
	bool		inThisNode(int, int) ;

	// input/output functions
	bool		save(ofstream&) ;
	bool		load(ifstream&, BasicLayoutComp*) ;

	// display the node
	bool		displayNode(void) ;
	bool		dump(void) ;
	bool		pan(int, int) ;
	bool		zoom(void) ;
} ;

class BasicEdge {
	EDGEID		id ;	// internal id, used by Swan, it can
				// also be used by annotated program
				// to access this edge
        LAYOUTCOMPID	lc_id ; // the layout component contains this
				// edge
	CURVE*		wpath ; // the physical position 
				// in Swan display window
        LABEL          	label ;  // text in the edge
	FLAG		active ; // display flag 

	INDEX		ed_id ;

    friend class BasicLC ;
    friend class BasicGraph ;

    public:

	CURVE*		ppath ; // the physical position of the edge
	CURVE*		rpath ;	// the relative position of the edge
				// in its layout component
	EDGEINFO	*info ; // display attributes
	INNODEID	s_node ; // the starting node of the edge
	INNODEID	e_node ; // the ending node of the edge

	// constructor and destructor
	BasicEdge(BasicNode*, BasicNode*, LABEL) ;
	BasicEdge() ;
	~BasicEdge() ;

	// access member functions
        bool         setLayoutComp(LAYOUTCOMPID) ;
        LAYOUTCOMPID    getLayoutComp(void) ;
        bool         setAttr(EDGEINFO*) ;
	EDGEINFO*	getAttr(void) ;
        bool         setColor(COLORTYPE) ;
        COLORTYPE       getColor(void) ;
        bool         setLineTH(int) ;
        int             getLineTH(void) ;
        bool         setLength(int) ;
        int             getLength(void) ;
	bool		setArrow(ARROWTYPE) ;
	bool		setRPath(CURVE*) ;
	CURVE*		getRPath(void) ;
	bool		setPPath(CURVE*) ;
	CURVE*		getPPath(void) ;
	bool		setWPath(CURVE*) ;
	CURVE*		getWPath(void) ;
	BasicNode*	getSNode(void) ;
	BasicNode*	getENode(void) ;
	LABEL		getLabel(void) ;	
	bool		setLabel(LABEL) ;	
	void		enActive(void) ;
	void		disActive(void) ;

	// edge states
	FLAG   		isDisplayed(void) ;

	// edge position
 	bool		calPosition(void) ;
	bool		createPPath(void) ;
	bool		calWPath(void) ;
	bool		calLabelWPos(void) ;
	bool 	_intersections(LOCATION, NODETYPE, BOXSIZE,
				       LOCATION, NODETYPE, BOXSIZE,
                          	       LOCATION&, LOCATION&) ;
	bool		getLast2Nodes(LOCATION&, LOCATION&) ;
	bool		getFirst2PPath(LOCATION&, LOCATION&) ;
	bool		getLast2PPath(LOCATION&, LOCATION&) ;

	// pick edge
	bool 	nearThisEdge(int, int) ;

	// input/output 
	bool		save(ofstream&) ;
	bool		load(ifstream&, BasicGraph*) ;

	// display the edge
	bool		displayEdge(void) ;
	bool		erase(void) ;
	bool		dump(void) ;
	bool		pan(int, int) ;
	bool		zoom(void) ;
	bool		displayArrow(void) ;
	void		displayLabel(void) ;
} ;

class BasicGraph {
        GRAPHID     id ;  

	LCLIST	    *lc_list ; // Layout Components List
	NODELIST    *node_list ; // Adjacency List
	EDGELIST    *edge_list ; // Edge List

	GRAPHINFO   *graph_info ; 
	NODEINFO    *node_info ;
	EDGEINFO    *edge_info ;
	LAYOUTCOMPID cur_lc ;
	FLAG	    active ;
	int	    nLayout ;
	FLAG	    fNewLayout ;
	FLAG	    fAutoRelayout ;

    public:   

	// operations on node_list
	BasicNode* searchNode(NODEID) ;
	BasicEdge* searchEdge(NODEID, NODEID) ;
	bool	deleteInNode(NODEID) ;

	// operations on neighbour node list
	friend struct _nb_node* searchNBNode(BasicNode*, NODEID) ;
	BasicEdge*	 getEdge(struct _nb_node*) ;
	bool		 insertNBNode(BasicNode*, 
				      struct _nb_node*) ;
	bool		 removeNBNode(BasicNode*, 
				      struct _nb_node*) ;

	// constructor and destructors
        BasicGraph(GRAPHID, NODETYPE) ;
	BasicGraph(GRAPHID) ;
        ~BasicGraph(void) ;

	// basic operations on nodes and edges
        bool insertNode(NODEID, LABEL) ;
        bool deleteNode(NODEID) ;
        bool insertEdge(NODEID, NODEID, LABEL) ;
        bool deleteEdge(NODEID, NODEID) ;

	// graph states
	bool isDisplayed(void) ;
	FLAG 	isNewLayout(void) ;
	FLAG	isAutoRelayout(void) ;

	// display
	bool	displayGraph(void) ;
	bool layoutGraph(void) ;
	bool	dump(void) ;
	bool pan(int, int) ;
	bool zoom(void) ;
	bool	setAllNodesPPos(void) ;

	// operations on layout component
	LC_ID 	     createLC(LC_ID, LAYOUTCOMPTYPE) ;
	bool	     removeLC(LC_ID) ;
	NODEID	     lastNodeLC(LAYOUTCOMPID) ;
	void         setFlagNode(LAYOUTCOMPID, INNODEID) ;
	bool	     setCurLC(LC_ID) ;
	LAYOUTCOMPID getCurLC(void) ;
	LAYOUTCOMPID getRootLC(void) ;
	LAYOUTCOMPID findCCA(LAYOUTCOMPID, LAYOUTCOMPID) ;
	BasicLayoutComp* searchLC(LC_ID) ;

	// access methods 
	GRAPHID	     getID(void) ;
	NODELIST*    getNodeList(void) ;
	LCLIST*	     getLCList(void) ;

	// graphical attributes
	bool      setDirected(FLAG) ;
	bool	     isDirected(void) ;
	bool	     setGraphInfo(GRAPHINFO*) ;
	GRAPHINFO*   getGraphInfo(void) ;
	bool	     setNodeInfo(NODEINFO*) ;
	NODEINFO*    getNodeInfo(void) ;
	bool	     setEdgeInfo(EDGEINFO*) ;
	EDGEINFO*    getEdgeInfo(void) ;
	bool	     setNodeType(NODETYPE) ;
	NODETYPE     getNodeType(void) ;
	bool	     setNodeColor(COLORTYPE) ;
 	COLORTYPE    getNodeColor(void) ;
	bool	     setNodeFilled(FLAG) ;
	FLAG	     getNodeFilled(void) ;
	bool	     setNodeWidth(int) ;
	int	     getNodeWidth(void) ;
	bool	     setNodeHeight(int) ;
	int	     getNodeHeight(void) ;
	bool	     setNodeLineTH(int) ;
	int	     getNodeLineTH(void) ;
	bool      setEdgeArrow(ARROWTYPE) ;
	bool      setEdgeColor(COLORTYPE) ;
	COLORTYPE    getEdgeColor(void) ;
	bool      setEdgeLength(int) ;
	int	     getEdgeLength(void) ;
	bool      setEdgeLineTH(int) ;
	int	     getEdgeLineTH(void) ;
	void	     setTravFlags(void) ;
	void 	     clearTravFlags(void) ;
	EDGELIST*    getEdgeList(void) ;
	LAYOUTCOMPTYPE getLayoutType(void) ;
	bool	     setLayoutType(LAYOUTCOMPTYPE) ;
	void 	     setNewLayoutFlag(FLAG) ;
	void	     setAutoRelayout(FLAG) ;
	void	     enActive(void) ;
	void 	     disActive(void) ;
	bool      resetActiveFlag(void) ;
	int	     getLayoutMode(void) ;
	bool      setLayoutMode(int) ;
	LOCATION     getPPos(void) ;
	bool	     setPPos(LOCATION) ;

	// pick edge
	bool	     edgeIn(int, int) ;

	// I/O 
	bool	     save(ofstream&) ;
	bool	     load(ifstream&) ;
} ;

class BasicLayoutComp {
	LC_ID		eid ;	// external id
	LAYOUTCOMPID	id ;	// id of this layout component for
				// internal and external access
	LAYOUTCOMPTYPE	type ;  // type of this layout component
	LAYOUTCOMPID	pid ;	// id of its parent layout component
	BasicGraph*	gid ;	// id of the graph contains this LC
	BasicNode	*pnode ; // the connection node in parent lc 
	BasicNode	*fnode ; // the connected by pnode  
	NODEID		pnode_id ;
	NODEID		fnode_id ;
	BOXSIZE		bbox ;	// bounding box of this component
	LOCATION	rpos ;	// relative position of this component
				// in its parent component
	LOCATION	ppos ;  // physical position of this component
				// in Swan display window
	LCNBLIST	*ch_lclist ; // children LC list
	NODELIST	*ch_nodelist ; // children node list

	LCINFO		*lc_info ;
	NODEINFO	*node_info ;
	EDGEINFO	*edge_info ;
	NODETYPE	node_type ;
	bool		active ;

	bool		rbus[MAXBUSLINE] ;
	bool		bbus[MAXBUSLINE] ;

    friend class BasicGraph ;
    friend class BasicEdge ;

    public:
	BasicLayoutComp(LC_ID,LAYOUTCOMPTYPE) ;
	BasicLayoutComp(void) ;
	~BasicLayoutComp(void) ;

	// access functions
	LC_ID		getEID(void) ;
	bool		setID(LAYOUTCOMPID) ;
	LAYOUTCOMPID	getID(void) ;
	bool		setPID(LAYOUTCOMPID) ;
	LAYOUTCOMPID	getPID(void) ;
	bool		setGID(BasicGraph*) ;
	BasicGraph*	getGID(void) ;
        bool         setRPos(LOCATION) ;
        LOCATION        getRPos(void) ;
        bool         setPPos(LOCATION) ;
        LOCATION        getPPos(void) ;
	bool		setBoxSize(BOXSIZE) ;
	BOXSIZE		getBoxSize(void) ;
	LAYOUTCOMPTYPE	getType(void) ;	
	bool		setType(LAYOUTCOMPTYPE) ;	
	LCNBLIST*	getChLCList(void) ;
	NODELIST*	getChNodeList(void) ;
	BasicNode*	getFNode(void) ;
	void		setFNode(BasicNode*) ;
	BasicNode*	getPNode(void) ;
	void		setPNode(BasicNode*) ;

	bool	     setNodeInfo(NODEINFO*) ;
	NODEINFO*    getNodeInfo(void) ;
	bool	     setEdgeInfo(EDGEINFO*) ;
	EDGEINFO*    getEdgeInfo(void) ;
	bool	     setNodeType(NODETYPE) ;
	NODETYPE     getNodeType(void) ;
	bool	     setNodeColor(COLORTYPE) ;
	bool	     setNodeFilled(FLAG) ;
	bool	     setNodeWidth(int) ;
	bool	     setNodeHeight(int) ;
	bool	     setNodeLineTH(int) ;
	bool      setEdgeArrow(ARROWTYPE) ;
	bool      setEdgeColor(COLORTYPE) ;
	bool      setEdgeLength(int) ;
	bool      setEdgeLineTH(int) ;

	// manipulation functions
	void		enActive(void) ;
	void		disActive(void) ;
	bool 	setChLCPPos(void) ;
	LAYOUTCOMPID	searchNBLC(BasicNode*) ; 
	int		getRBus(void) ;
	int		getBBus(void) ;
	bool		inThisLC(int, int, PICKTYPE) ;

	// LC states
	bool		isDisplayed(void) ;

	// display layout component
	bool	        clearLC() ;
	bool	        displayLC() ;
	bool		dump() ;
	
	// input/output
	bool		save(ofstream&, int) ;
	bool		load(ifstream&, int, BasicLayoutComp*) ;
	bool		restorePFNode(void) ;
} ;
	
// typedef DoubleList<BasicGraph*>	GRAPHLIST ;

/*--------------- Constants ---------------*/

const LOCATION UnsetPos = {-1, -1} ; 
const BOXSIZE  EmptyBox = { 0,  0} ;

/*--------------- Global Variable Declarations ---------------*/

extern GRAPHLIST* ggl ; // Graph List 
extern BasicLayoutComp*	_sw_root_lc ; // the root layout comp
extern char* color_table[] ;

extern int fPause ;
extern int fStepOn ;
extern int fNextStep ;

// current global graphic attributes
extern FLAG 		_sw_cur_fDirected ;
extern NODETYPE		_sw_cur_nodeType ;
extern COLORTYPE	_sw_cur_nodeColor ;
extern FLAG		_sw_cur_nodeFilled ;
extern int		_sw_cur_nodeWidth ;
extern int		_sw_cur_nodeHeight ;
extern int		_sw_cur_nodeLineTH ;
extern int		_sw_cur_edgeLength ;
extern int		_sw_cur_edgeLineTH ;
extern ARROWTYPE	_sw_cur_edgeArrow ;
extern COLORTYPE	_sw_cur_edgeColor ;
extern int		_sw_nodepicked ;
	
// system internal switches
extern FLAG 	_sws_autoredraw ;
extern FLAG     _sws_swapnode ;
extern FLAG     _sws_movenode ;
extern FLAG	_sws_insertnode ;
extern FLAG	_sws_deletenode ;
extern FLAG	_sws_insertedge ;
extern FLAG	_sws_deleteedge ;

/*---------------- Function Prototypes ---------------*/

// Swan internal functions
bool _sw_init(void) ;
bool _sw_quit(void) ;
bool _sw_wininit(GS_MAIN_ARGS_F) ;
void _sw_set_allgraphs_pos(void) ;
BasicGraph* _sw_searchgraph(GRAPHID) ;
void _sw_check_buttons(void) ;

// Swan layout tools 
bool _sw_segedgepos(LOCATION, LOCATION, BOXSIZE, EDGEINFO*&) ;
bool _sw_normalizeRPos(BasicGraph*) ;

// Swan exception handling
void out_of_store(void) ;

/*--------------- Other included files ---------------*/

#include "control.h"
#include "swg.h"
#include "geotools.h"
#include "clbnode.h"
#include "clbedge.h"
#include "clblay.h"
#include "clbgraph.h"
#include "clnbnode.h"
#include "clpoint.h"

/************************************************************
 * End of swan.h				            *
 ************************************************************/
#endif
