/************************************************************
 * swg.h -- header file for Swan Graphics Interface         *
 * Jun Yang, November 17, 1994				    * 
 ************************************************************/

/*--------------- Constants ---------------*/

// numbers
#define MAXPRIMNUM	2048	
#define SWG_SHOWWIN	"Help"

// types of graphic primitives
#define LINE		1
#define RECTANGLE	2
#define ELLIPSE		3
#define STRING		4

// types of rectangles
#define FRAMEDRECT	1
#define FILLEDRECT	2

// types of ellipses
#define FRAMEDELLIPSE	1
#define FILLEDELLIPSE	2

// types of picked primitives 
#define PICKALL		0
#define PICKNODE	1
#define PICKEDGE	2
#define PICKGRAPH	3

// specifications on width/height of nodes
#define _SW_MINEDGELENGTH	8
#define _SW_MAXEDGELENGTH	50
#define _SW_MINNODEWIDTH	15
#define _SW_MAXNODEWIDTH	50
#define _SW_MINNODEHEIGHT	8
#define _SW_MAXNODEHEIGHT	30

/*--------------- Types --------------*/

typedef int	LINETYPE ;
typedef int	RECTTYPE ;
typedef int	ELLIPSETYPE ;
typedef int	ARROWTYPE ;
typedef int	COLORTYPE ;

typedef struct {
	char	*name ;
	BOXSIZE	size ;
} FONTTYPE ;

/*--------------- External Variables ---------------*/

extern int 	sc_x_origin ;
extern int 	sc_y_origin ;
extern float 	zoom_factor ;

extern LINETYPE	swg_currentLineType ;
extern int	swg_currentRectType ;
extern int	swg_currentEllipseType ;
extern ARROWTYPE swg_currentArrowType ;

extern LOCATION		swg_currentPos ;
extern COLORTYPE   	swg_currentColor ;
extern int	    	swg_currentsize ;
extern FONTTYPE    	swg_currentFont ;

extern char	*color_table[] ;
extern FONTTYPE font_table[] ;

extern BasicGraph*	_sw_pick_graph ;
extern BasicNode*	_sw_pick_node ;
extern BasicEdge*	_sw_pick_edge ;
extern BasicNode*	_sw_pick_node1 ;
extern BasicNode*	_sw_pick_node2 ;
extern LOCATION		_sw_pick_pos ;
extern char		_sw_pick_string[80] ;

extern char		InputTextString[80] ;
extern char		OutputTextString[80] ;

/*--------------- Function Prototypes ---------------*/

// Drawing functions
bool   swg_drawline(LOCATION, LOCATION) ;
bool   swg_drawline(int, int, int, int) ;
bool   swg_drawcurve(CURVE*) ;
bool   swg_drawrectangle(int, int, int, int, RECTTYPE) ;
bool   swg_drawellipse(int, int, int, int, ELLIPSETYPE) ;
bool   swg_drawstring(int, int, char*) ;

// Pick functions
bool swg_pick(int, int, PICKTYPE) ;

// Erase functions
bool swg_releaseEntry(int) ;
bool swg_erase(int) ;
bool swg_clear(LOCATION, BOXSIZE) ;
bool swg_clearpath(CURVE*) ;
bool swg_clearEntry(void) ;

// Attribute functions
bool swg_setcurrentlinetype(LINETYPE) ;
bool swg_setcurrentrecttype(int) ;
bool swg_setcurrentellipsetype(int) ;

bool swg_setcurrentpos(LOCATION) ;
bool swg_setcurrentarrowtype(ARROWTYPE) ;
bool swg_setcurrentsize(int) ;
bool swg_setcurrentcolor(COLORTYPE) ;
bool swg_setcurrentfont(FONTTYPE) ;

bool swg_setcolor(INDEX, COLORTYPE) ;
int getPrimNodeEntry();
bool swg_refresh() ;
bool swg_dump() ;
bool swg_pan(int, int) ;
bool swg_zoom() ;
bool swg_swapnode(BasicNode*, BasicNode*) ;
bool swg_movenode(BasicNode*, int, int) ;  

bool	swg_savegraph(char*, BasicGraph*) ;
bool	swg_loadgraph(char*) ;

/************************************************************
 * End of swg.h						    *
 ************************************************************/
